Capability Query Library
===============================================

This is query library operating with capability and resource ontologies. This library provides a higher level API to interact with these ontologies. Relates to EU Hz-2020 project ReCaM <http://www.recam-project.eu/>.

Repository location
--------------------
The main storage for this GIT repository can be found from:
<https://bitbucket.org/nikkeSi/capabilityquerylib>

Administrator of this GIT repository is:  
Tampere University (TAU) / 
Faculty of Engineering and Natural Sciences (ENS) / 
Automation Technology and Mechanical Engineering (ATME) / Manufacturing and Production Automation /
Niko Siltala  
niko.siltala@tuni.fi  
+358-40-536 6017  

Authors
-------
The main authors for this repository are:

* Otto Hylli, otto.hylli@tuni.fi


Software Versions used for the application
------------------
Software | Version | Additional information
--- | --- | ---
OpenJDK 64-Bit	| 1.8
Oracle Java JDK	| 1.8
Maven		| 3.0.x
Jena		 | 3.1.1
Openllet		 | 2.6.0
[Resource description API](https://bitbucket.org/nikkeSi/rd_api) | 0.2.0-DEV


Prerequisites
---------------
### Software needs to be installed. 

Software | Version
-------- | -------
Java JDK	| 1.8
Maven		| 3.0

FOLDER STRUCTURE:
-----------------


Folder | Description 
--- | ---
query_templates/		| SPARQL template queries used by the application
query_example | some SPARQL example queries that can be executed with the application
rules | some example SPIN rules used by the application. note the application does not directly use the rules in the directory.
src/main/java/			| Java source codes
src/test/java/			| Test files
target/				| Build and package files
pom.xml				| Maven targets
parameterrules.ttl | ontology file containing SPIN rules for calculating combined capability parameters. Some of these are available as text files in the rules directory.
cql.bat			| Windows bat file to start application
cql.sh			| *nix start script 


Compilation:
------------
Application will be compiled, deployed, etc. with Maven.  
Make sure Maven version is correct and that Maven uses right java version.
```shell
mvn -v
```

Compilations can be performed executing:

```shell
mvn compile			-> Compiles
mvn package			-> Compiles, tests and makes executable jar package and zip distribution package (see assembly:single)
mvn package -DskipTests       -> Same as above but skips tests
mvn install           -> Compiles, tests and makes jar package which is then copied to local Maven repository
mvn assembly:single    -> creates a zip archive from the compiled project and javadoc (package and javadoc:javadoc have to be executed before)
mvn deploy            -> Deploys packaged JARs to our public Maven repository
mvn javadoc:javadoc     -> creates API documentation
mvn site:site            -> Creates web sites for the project
mvn site:deploy       -> Deploys site to our public documentation server.
mvn clean			-> Cleans project folders from compiled files
```

## Usage as a dependency in another maven project

To use the capabilityQueryLibrary in another project that uses maven add our maven repository and this project as a dependency in your own application's pom.xml:

```xml
<project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
   xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/xsd/maven-4.0.0.xsd">
   
    <!-- your maven stuff -->
    <dependencies>
        <dependency>
            <groupId>fi.tut.mei</groupId>
            <artifactId>capabilityQueryLib</artifactId>
            <version>1.3.0-DEV</version>
        </dependency>
    <!-- your other dependencies --> 
    </dependencies>
    <repositories>
      <repository>
        <id>resourcedescription</id>
        <url>http://resourcedescription.tut.fi/maven</url>
      </repository>
   </repositories>
   <!-- your maven stuff -->
</project>
```

Note check the correct version to use in your pom.xml from this project's pom.xml.

## Using the library

To use the library in your application create a fi.tut.mei.capabilityQueryLib.Client object for the ontology containing resource instances you want to query. The client offers different methods for querying the ontology. You can check the available methods and other classes from the javadoc which you can generate with maven:

```shell
mvn javadoc:javadoc 
```

The documentation will be created into the target/site directory. Below is a simple usage example that determines the combined capabilities that can be formed from the capabilities Moving and FingerGrasping. To see more usage examples look at the source code of the fi.tut.mei.capabilityQueryLib.App class which implements a command line interface for the library.

```java
import java.util.ArrayList;
import java.util.List;

import fi.tut.mei.capabilityQueryLib.Client;
import fi.tut.mei.capabilityQueryLib.model.Capability;

public class App {
    public static void main( String[] args ) {
        // client that uses default ontology from server
        Client client = new Client();
        List< Capability > capabilities = new ArrayList<>();
        capabilities.add( client.getCapability( "Moving" ));
        capabilities.add( client.getCapability( "FingerGrasping" ));
        // note getCapability returns null if there is no such capability so you should check for that
        List< Client.CombinedCapabilityInfo > results = client.getCombinedCapabilities( capabilities );
        for ( Client.CombinedCapabilityInfo result : results ) {
            System.out.println( result.getCombinedCapability() );
        }
    }
}
``` 
 
Command line usage
-------------------

The library offers a command line interface for querying the ontology. You can create anexecutable jar with the following command:

```shell
mvn package 
```

This copies the project's dependencies to a lib directory which is included to the class path of the created jar. You can then execute the created jar with the help of the included command line scripts cql.bat for Windows and cql.sh for unix based systems like Linux or Mac. By default the used ontology file is downloaded from a server but you can use your own ontology file. Below are some usage examples:

```shell
# print usage instructions
cql
cql -h
# Information about which combined capabilities can consist of capability ToolHolding 
cql -sc ToolHolding
# Information about what capabilities can form the combined capability Milling
cql -c Milling
# Show parameters related to capability Moving.
cql -cp Moving
# Show combined capabilities that can be formed from capabilities Moving and FingerGrasping
cql -cc Moving,FingerGrasping
# Show information about device combination Makino_combination_1 including combined capabilities and parts
cql -d Makino_combination_1
# show information about device combination from ontology file named myontology.owl
# the option -o can be used with all of the commands
cql -d Makino_combination_1 -o myontology.owl
# Save the combined capabilities of a device combination to a ontology file (a new file or an existing file)
cql -d Makino_combination_1 -sd combined.n3
# Calculate the combined capabilities and their parameters for all device combinations and save to file
cql -sa combinations.ttl
```

## Additional information

Additional information about CQL such as an architecture overview is available in the CQL [wiki](https://bitbucket.org/nikkeSi/capabilityquerylib/wiki/Home).